#include <boost/thread/condition_variable.hpp>
#include <boost/thread/mutex.hpp>
#include <boost/thread/locks.hpp>
#include <boost/bind.hpp>
#include <boost/shared_ptr.hpp>
#include <boost/thread.hpp>
#include <cstdlib>
#include <iostream>
#include <unistd.h>
#include <vector>

extern "C" {
#include <pipeline.h>
}

#define NUM_THREADS 2

static boost::mutex mutex, cout_mutex;
static boost::condition_variable cond;
static bool has_work = false;

static void worker_thread()
{
    while (true) {
        {
            boost::unique_lock<boost::mutex> lock(mutex);
            while (!has_work)
                cond.wait(lock);
            has_work = false;
        }
        cond.notify_all();

        pipeline *pl = pipeline_new();
        pipeline_command_args(pl, "echo", "foo", NULL);
        pipeline_command_args(pl, "sleep", "2", NULL);
        pipeline_start(pl);
        if (pipeline_wait(pl) != EXIT_SUCCESS)
            abort();

        boost::unique_lock<boost::mutex> lock(cout_mutex);
        std::cout << "CONSUMED SOME WORK" << std::endl;
    }
}

int main(int argc, char **argv)
{
    std::vector<boost::shared_ptr<boost::thread> > threads;
    threads.reserve(NUM_THREADS);
    for (int i = 0; i < NUM_THREADS; ++i)
        threads.push_back(boost::shared_ptr<boost::thread>(new boost::thread(boost::bind(&worker_thread))));

    while (true) {
        {
            boost::unique_lock<boost::mutex> lock(mutex);
            while (has_work)
                cond.wait(lock);
            has_work = true;
        }
        cond.notify_one();

        boost::unique_lock<boost::mutex> lock(cout_mutex);
        std::cout << "PRODUCED SOME WORK" << std::endl;

        sleep(1);
    }

    return EXIT_SUCCESS;
}
